import 'dart:io';

import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/streak_congratulation_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_cover_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_flashcard_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_write_mode_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/conjugation_meta_data_card.dart';
import 'learn_or_repeat_route.dart';

/// The Route that allows you to test your knowledge of the verb that you want
/// to learn
class VerbIndicativeTestRoute extends StatelessWidget {
  static const routeName = '/verbIndicativeTestRoute';

  final SettingsController _settingsController;

  const VerbIndicativeTestRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          themeMode: _settingsController.themeMode,
          theme: CustomTheme.lightTheme,
          darkTheme: ThemeData.dark(),
          debugShowCheckedModeBanner: false,
          home: Scaffold(
            appBar: AppBar(
              leading: IconButton(
                  key: const Key('BackKey'),
                  onPressed: () {
                    Navigator.pop(context);
                  },
                  icon: const Icon(Icons.arrow_back)),
              actions: [
                _repeatIcon(context),
                _homeIcon(context),
                _settingsIcon(context)
              ],
              title: _titleCell(context),
            ),
            body: Container(child: _formBody(context)),
            floatingActionButton: _evaluateButton(context),
          ),
        );
      },
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  IconButton _homeIcon(BuildContext context) {
    return IconButton(
        key: const Key('home_button'),
        onPressed: () {
          final VerbTestModel testModel =
              Provider.of<VerbTestModel>(context, listen: false);
          testModel.reset();
          Navigator.restorablePopAndPushNamed(
              context, LearnOrRepeatRoute.routeName);
        },
        icon: const Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    return IconButton(
        key: const Key('back_button'),
        onPressed: () {
          final VerbTestModel testModel =
              Provider.of<VerbTestModel>(context, listen: false);
          testModel.reset();
          final StudyModeState studyModeState =
              Provider.of<StudyModeState>(context, listen: false);
          final StudyMode? studyMode = studyModeState.currentStudyMode;
          if (studyMode != null) {
            switch (studyMode) {
              case StudyMode.flashCardMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyFlashcardModeRoute.routeName);
                break;
              case StudyMode.coverMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyCoverModeRoute.routeName);
                break;
              case StudyMode.writtenMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyWriteModeRoute.routeName);
                break;
            }
          } else {
            Navigator.popAndPushNamed(context, '/verbIndicativeStudyRoute');
          }
        },
        icon: const Icon(Icons.refresh));
  }

  Widget _titleCell(BuildContext context) {
    return Wrap(children: const [
      Text(
        'Test je kennis',
        textAlign: TextAlign.center,
      ),
    ]);
  }

  FloatingActionButton _evaluateButton(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);

    return FloatingActionButton(
      key: toStreakCongratulationButtonKey(testModel.infinitive),
      child: const Icon(Icons.arrow_forward),
      onPressed: () {
        if (testModel.isComplete()) {
          _settingsController.updateStreakEngine();
          Navigator.pushNamed(context, StreakCongratulationRoute.routeName);
        }
      },
    );
  }

  Widget _formBody(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return SafeArea(
      child: Scaffold(
          body: Column(
        children: [
          _conjugationMetaDataCard(context),
          const Divider(),
          Expanded(
            child: ListView.builder(
                key: Key('scroll_test_${testModel.infinitive}'),
                itemCount: testModel.expected.length,
                scrollDirection: Axis.vertical,
                controller: ScrollController(),
                physics: const ScrollPhysics(
                  parent: AlwaysScrollableScrollPhysics(),
                ),
                itemBuilder: (BuildContext context, int index) {
                  return Padding(
                    padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
                    child: Card(
                      elevation: 5.0,
                      child: ListTile(
                        visualDensity: VisualDensity.comfortable,
                        contentPadding: const EdgeInsets.fromLTRB(15, 5, 30, 5),
                        dense: true,
                        title: Row(
                          children: [
                            _pronounCell(context, index),
                            Expanded(child: _verbCell(context, index)),
                          ],
                        ),
                      ),
                    ),
                  );
                }),
          ),
        ],
      )),
    );
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  Widget _pronounCell(BuildContext context, int person) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return PronounField(
        targetLanguage: testModel.targetLanguageIso,
        fullPronoun: testModel.pronouns[person],
        expectedIndicative: testModel.expected[person]);
  }

  Widget _verbCell(BuildContext context, int person) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return SizedBox(
      height: 40,
      child: Container(
        alignment: Alignment.center,
        child: FocusEnabledTextField(person: person, testModel: testModel),
      ),
    );
  }

  static Key testFieldKey(String infinitive, int person) {
    return Key('test_${infinitive}_$person');
  }

  static Key toStreakCongratulationButtonKey(String infinitive) {
    return Key('congratulate_$infinitive');
  }
}

class FocusEnabledTextField extends StatefulWidget {
  const FocusEnabledTextField(
      {Key? key, required this.person, required this.testModel})
      : super(key: key);

  final int person;
  final VerbTestModel testModel;

  @override
  State<FocusEnabledTextField> createState() {
    return FocusEnabledTextFieldState();
  }
}

class FocusEnabledTextFieldState extends State<FocusEnabledTextField> {
  late FocusNode myFocusNode;

  @override
  void initState() {
    super.initState();
    myFocusNode = FocusNode(onKeyEvent: (FocusNode node, KeyEvent event) {
      if (Platform.isIOS) {
        return KeyEventResult.skipRemainingHandlers;
      } else {
        return KeyEventResult.handled;
      }
    });
  }

  @override
  void dispose() {
    myFocusNode.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      key: testFieldKey(widget.testModel.infinitive, widget.person),
      focusNode: myFocusNode,
      obscureText: false,
      autofocus: true,
      showCursor: true,
      decoration: const InputDecoration(
        border: OutlineInputBorder(),
      ),
      readOnly: false,
      onChanged: (String inputValue) {
        widget.testModel.add(inputValue, widget.person);
        myFocusNode.requestFocus();
      },
    );
  }

  static Key testFieldKey(String infinitive, int person) {
    return Key('test_${infinitive}_$person');
  }
}
